<?php

namespace Drupal\webform_protected_downloads\Entity;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;

/**
 * Defines the Webform protected downloads entity.
 *
 * @ingroup webform_protected_downloads
 *
 * @ContentEntityType(
 *   id = "webform_protected_downloads",
 *   label = @Translation("Webform protected downloads"),
 *   base_table = "webform_protected_downloads",
 *   entity_keys = {
 *     "id" = "id",
 *   },
 * )
 */
class WebformProtectedDownloads extends ContentEntityBase {

  /**
   * Return WebformSubmission.
   *
   * @return \Drupal\webform\Entity\WebformSubmission|null
   *   Return WebformSubmission if available.
   */
  public function getWebformSubmission() {
    if ($first = $this->get('webform_submission')->first()) {
      /** @var \Drupal\Core\Entity\Plugin\DataType\EntityReference $reference */
      $reference = $first->get('entity');
      return $reference->getValue();
    }
  }

  /**
   * Get hash.
   *
   * @return null|string
   *   Return hash if exists.
   */
  public function getHash() {
    return $this->hash->value;
  }

  /**
   * Get webform handler.
   *
   * @param bool $verify_file
   *   Verify the webform handler still references the same file entity as the
   *   protected download entity. Defaults to TRUE.
   *
   * @return \Drupal\webform\Plugin\WebformHandlerInterface|null
   *   Return handler if exists and enabled.
   */
  public function getHandler($verify_file = TRUE) {
    // Get WebformSubmission.
    if ($webformSubmission = $this->getWebformSubmission()) {
      // Check handler is active.
      if ($webform = $webformSubmission->getWebform()) {
        try {
          $handler = $webform->getHandler($this->handler_id->value);
          if ($handler->isEnabled()) {
            // Check that the handler still references the same file, unless
            // that verification step can be skipped.
            if (!$verify_file || $this->file->target_id == current($handler->getSetting('protected_file'))) {
              return $handler;
            }
          }
        }
        catch (PluginNotFoundException $e) {
          // Referenced handler is no longer available.
        }
      }
    }
  }

  /**
   * Check if link is active.
   *
   * @return bool
   *   Return bool.
   */
  public function isActive() {
    return (bool) $this->active->value;
  }

  /**
   * Check if link is only one time usable.
   *
   * @return bool
   *    Return bool.
   */
  public function isOneTimeLink() {
    return (bool) $this->onetime->value;
  }

  /**
   * Returns the link URL for downloading this protected download.
   *
   * @return \Drupal\Core\GeneratedUrl|null
   *    The link URL if available.
   */
  public function getUrl() {
    $hash = $this->getHash();
    if (!is_null($hash)) {
      $url = Url::fromRoute('webform_protected_downloads.download', [
        'hash' => $hash,
      ], [
        'absolute' => TRUE,
      ]);
      return $url->toString();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['webform_submission'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Webform Submission'))
      ->setSetting('target_type', 'webform_submission')
      ->setRequired(TRUE);

    $fields['handler_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Handler ID'))
      ->setRequired(TRUE);

    $fields['file'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('File'))
      ->setSetting('target_type', 'file')
      ->setRequired(TRUE)
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED);

    $fields['hash'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Hash'))
      ->setRequired(TRUE);

    $fields['active'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Active'))
      ->setDefaultValue(FALSE);

    $fields['expire'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Expire'))
      ->setRequired(TRUE);

    $fields['onetime'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Onetime'))
      ->setDefaultValue(FALSE)
      ->setRequired(TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);
  }

}
